<?php

namespace vtbpayments;

class VtbApi
{
    private static $tokenUrl;

    private static $orderUrl;

    private static $mode;

    private static $clientMerchant;

    private static $clientData = [
        'grant_type' => 'client_credentials'
    ];

    private static $modeUrls = [
        'PROD' => [
            'TOKEN' => 'https://open.api.vtb.ru:443/passport/oauth2/token',
            'ORDER' => 'https://gw.api.vtb.ru/openapi/smb/efcp/e-commerce/v1/orders'
        ],
        'TEST' => [
            'TOKEN' => 'https://auth.bankingapi.ru/auth/realms/kubernetes/protocol/openid-connect/token',
            'ORDER' => 'https://hackaton.bankingapi.ru/api/smb/efcp/e-commerce/api/v1/orders'
        ]
    ];

    /**
     * @param $clientId
     * @param $clientSecret
     * @param $test
     */
    public function __construct(
        $clientId = false,
        $clientSecret = false,
        $clientMerchant = false,
        $test = false
    )
    {
        self::$clientData['client_id'] = $clientId;
        self::$clientData['client_secret'] = $clientSecret;
        self::$clientMerchant = $clientMerchant;

        self::$mode = $test ? 'TEST' : 'PROD';

        self::$tokenUrl = self::$modeUrls[self::$mode]['TOKEN'];
        self::$orderUrl = self::$modeUrls[self::$mode]['ORDER'];
    }

    /**
     * @return array
     */
    private function getAccessToken()
    {
        $headers[] = 'Content-Type: application/x-www-form-urlencoded';
        if (self::$clientMerchant) {
            $headers[] = 'Merchant-Authorization: ' . self::$clientMerchant;
        }

        $result = $this->sendRequest(
            self::$tokenUrl,
            http_build_query(self::$clientData),
            $headers
        );

        return $result;
    }

    /**
     * @return bool|array
     */
    private function getAutorizationHeaders()
    {
        $result = false;
        $token = $this->getAccessToken();

        if (is_array($token) && isset($token['access_token'])) {
            $result = [
                'X-IBM-Client-Id: ' . explode('@', self::$clientData['client_id'])[0],
                'Authorization: ' . $token['token_type'] . ' ' . $token['access_token'],
            ];
        }

        return $result;
    }

    /**
     * @param string $orderId
     * @param string $email
     * @param int $create
     * @param float $amount
     * @return bool|array
     */
    public final function getOrderLink(
        string $orderId,
        string $email,
        int    $create,
        float  $amount
    )
    {
        $result = false;
        $args = [];
        $headers = $this->getAutorizationHeaders();

        if ($headers) {
            $headers[] = 'Content-Type: application/json';
            if (self::$clientMerchant) {
                $headers[] = 'Merchant-Authorization: ' . self::$clientMerchant;
            }

            $timestamp = $create + 86400; // +1d
            $expire = date('Y-m-d\TH:i:s.v\Z', $timestamp); // ISO 8601

            $args = [
                'orderId' => $orderId,
                'orderName' => 'Заказ #' . $orderId,
                'customer' => [
                    'email' => $email
                ],
                'amount' => [
                    'value' => $amount,
                    'code' => 'RUB'
                ],
                'expire' => $expire,
                'returnUrl' => $this->getOrderUrl()
            ];

            $result = $this->sendRequest(
                self::$orderUrl,
                json_encode($args),
                $headers
            );
        }

        return $result;

    }

    /**
     * @param string $orderId
     * @return bool|array
     */
    public final function getOrderInfo(string $orderId)
    {
        $result = false;
        $headers = $this->getAutorizationHeaders();

        if ($headers) {
            $headers[] = 'Content-Type: application/json';
            if (self::$clientMerchant) {
                $headers[] = 'Merchant-Authorization: ' . self::$clientMerchant;
            }

            $result = $this->sendRequest(
                self::$orderUrl . '/' . $orderId,
                json_encode([]),
                $headers,
                'GET'
            );
        }

        return $result;
    }

    /**
     * @param int $order
     * @return string
     */
    private function getOrderUrl()
    {
        $protocol = (empty($_SERVER['HTTPS']) ? 'http' : 'https');
        $host = $_SERVER['HTTP_HOST'];
        $orderPage = '/index.php?dispatch=payment_notification.check&payment=vtbpayments';

        return "{$protocol}://{$host}/{$orderPage}";
    }

    /**
     * @param string $url
     * @param array $data
     * @param array $headers
     * @param string $method
     * @return array
     */
    public final function sendRequest(
        $url,
        $data = [],
        $headers = [],
        $method = 'POST'
    )
    {
        $curl = curl_init();

        switch ($method) {
            case 'POST':
                curl_setopt($curl, CURLOPT_POST, 1);
                curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
                break;
            case 'PUT':
                curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'PUT');
                curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
                break;
            default:
                $url = sprintf('%s?%s', $url, is_array($data) ? http_build_query($data) : '');
        }

        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

        $result = curl_exec($curl);

        if ($result) {
            $result = json_decode($result, true);
        }

        curl_close($curl);

        return $result;
    }
}